#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <fcntl.h>

#include <time.h> // time

#include <dirent.h> // opendir


#ifndef WIN32
#include <netdb.h> // gethostbyname
#endif

#include <extdll.h>
#include <sdk_util.h>
#include <meta_api.h>
#include <cbase.h>

#include "webmod.h"

MYMUTEX_T mutex; // mutex ; not currently used
int m_connfd; // global socket file descriptor ; unused
char m_clientip[16]; // global client ip ; unused
THREAD_T mythread; // global thread, used by server main thread
THREAD_T srede; // client handling threads
MYSOCK_T sock; // global socket
bool freebsd;

void startThread(void)
{
	MYTHREAD_CREATE(&mythread,theThread,NULL);
LOG_DBG_SERVER(("HTTP server thread created"));
}

void theThread(void *data)
{
MYSOCK_ADDRTYPE_IN sin;
MYSOCK_LENT socklen;
int connfd;
int res;
threaddata_t *tdata;

	MYSOCK_START();

#ifdef WIN32

	memset(&sin,0x0,sizeof(sin));

#endif

	setAddrIn(&sin,false);


	if((MYSOCK_NEW(sock))==INVALID_SOCKET)
	{
		LOG_MESSAGE(PLID,"Could not create socket.");
		exit(1);
	}


	// get rid of eaddrinuse (reuse port immediately after a crash)
	MYSOCK_SETOPT(sock,SO_REUSEADDR);


	// bind the socket ; also detect if os is linux or freebsd (linux emulation)

	// (under freebsd, bind fails if 'sin' was not filled of 0x00 before call)

	freebsd = false;

	res = MYSOCK_BIND(sock,sin);



#ifndef WIN32

	if(res == SOCKET_ERROR)

	{

		freebsd = true;

		memset(&sin,0x0,sizeof(sin));

		setAddrIn(&sin,true);

		res = MYSOCK_BIND(sock,sin);

	}
#endif


	if(res==SOCKET_ERROR)
	{
		LOG_MESSAGE(PLID,"Could not bind socket to ip %s (%d:%d)",!strcmp(CVAR_GET_STRING("ip"),"localhost")?"0.0.0.0":CVAR_GET_STRING("ip"), sin.sin_addr.s_addr,htons(sin.sin_port));

		switch(errno)
		{
		case EBADF: LOG_MESSAGE(PLID,"ebadf"); break;
		case EACCES: LOG_MESSAGE(PLID,"eaccess"); break;
		case ENOTSOCK: LOG_MESSAGE(PLID,"enotsock"); break;
		case EADDRINUSE: LOG_MESSAGE(PLID,"eaddrinuse"); break;
		case EINVAL: LOG_MESSAGE(PLID,"einval"); break;
		case EROFS: LOG_MESSAGE(PLID,"erofs"); break;
		case EFAULT: LOG_MESSAGE(PLID,"efault"); break;
		case ENAMETOOLONG: LOG_MESSAGE(PLID,"enametoolong"); break;
		case ENOENT: LOG_MESSAGE(PLID,"enoent"); break;
		case ENOMEM: LOG_MESSAGE(PLID,"enomem"); break;
		case ENOTDIR: LOG_MESSAGE(PLID,"enotdir"); break;
		case ELOOP: LOG_MESSAGE(PLID,"eloop"); break;
		default: LOG_MESSAGE(PLID,"UNKNOWN ERROR : errno=%d",errno); break;
		}
		exit(1);
	}


	// listen on our server (tcp) port
	if((MYSOCK_LISTEN(sock,1024))==SOCKET_ERROR)
	{
		LOG_MESSAGE(PLID,"Could not make the socket listen.");
		exit(1);
	}

	// initialize our mutex (unused)

	MYMUTEX_INIT(&mutex);


	// do not crash on connection problem
	signal(SIGPIPE,SIG_IGN);
	signal(SIGINT,cleanClose);

	LOG_MESSAGE(PLID,"Now ready : listening...");

	while(1)
	{
LOG_DBG_SERVER(("Ready for a new connection"));

		socklen=sizeof(sin);
		connfd = MYSOCK_ACCEPT(sock,sin,&socklen);

		if((int)connfd==(int)INVALID_SOCKET)
		{
			LOG_MESSAGE(PLID,"Error while accepting connection");
			continue;
		}

LOG_DBG_SERVER(("New connection ! (%d)",connfd));

/*
		m_connfd = connfd;
		memset(m_clientip,0,16);
		strncpy(m_clientip,inet_ntoa(sin.sin_addr),15);
*/

LOG_DBG_SERVER(("Preparing shared data : %d bytes",sizeof(threaddata_t)));

		tdata = (threaddata_t *)malloc(sizeof(threaddata_t));
LOG_DBG_SERVER(("Mem location : 0x%X",tdata));

		if(!tdata)
		{
			LOG_MESSAGE(PLID,"ERROR: malloc() failed on connection attempt");
			continue;
		}

		tdata->connfd = connfd;
		memset(tdata->clientip,0,16);
		strncpy(tdata->clientip,inet_ntoa(sin.sin_addr),15);

LOG_DBG_SERVER(("Creating thread to handle connection"));

		//MYTHREAD_CREATE(&(tdata->threadid),connectHandle,tdata);
		MYTHREAD_CREATE(&srede,connectHandle,tdata);

	}


	// never happens btw...
	MYSOCK_STOP(sock);
}

int sendHeaders(int connfd, httpresp_t *resp, httpquery_t *query, bool isDynamic)
{
char str[16];

	if(query->location[0]!='\0' || resp->location[0]!='\0')
		resp->code = 302;

	sprintf(str,"HTTP/1.1 %d ",resp->code);

	MYSOCK_WRITE(connfd,str);

	switch(resp->code)
	{
	case 200: MYSOCK_WRITE(connfd,"OK\n"); break;
	case 302: MYSOCK_WRITE(connfd,"Found\n"); break;
	default: MYSOCK_WRITE(connfd,"OK\n"); break;
	}

	MYSOCK_WRITE(connfd,"Server: WebMod/" VVERSION " ");
#ifdef WIN32
	MYSOCK_WRITE(connfd,"(Win32)\n");
#else
	if(!freebsd)
		MYSOCK_WRITE(connfd,"(Linux)\n");
	else
		MYSOCK_WRITE(connfd,"(FreeBSD)\n");
#endif
	MYSOCK_WRITE(connfd,"X-Powered-By: dJeyL :-)\n");
	if(isDynamic)
		MYSOCK_WRITE(connfd,"Cache-Control: no-store, no-cache, must-revalidate\n");
	MYSOCK_WRITE(connfd,resp->cookies);
	MYSOCK_WRITE(connfd,query->sendcookies);
	MYSOCK_WRITE(connfd,"Connection: close\n");

	if(query->location[0] != '\0')
	{
		MYSOCK_WRITE(connfd,"Location: ");
		MYSOCK_WRITE(connfd,query->location);
		MYSOCK_WRITE(connfd,"\n");
	}
	else if(resp->location[0] != '\0')
	{
		MYSOCK_WRITE(connfd,"Location: ");
		MYSOCK_WRITE(connfd,resp->location);
		MYSOCK_WRITE(connfd,"\n");
	}

	if(query->contenttype[0]!='\0')
	{
		MYSOCK_WRITE(connfd,"Content-type: ");
		MYSOCK_WRITE(connfd,query->contenttype);
		MYSOCK_WRITE(connfd,"\n\n");
	}
	else if(resp->ct[0] != '\0')
	{
		MYSOCK_WRITE(connfd,"Content-type: ");
		MYSOCK_WRITE(connfd,resp->ct);
		MYSOCK_WRITE(connfd,"\n\n");
	}
	else
		MYSOCK_WRITE(connfd,"Content-type: text/html\n\n");

	return 1;
}

void clientHandle(int connfd, httpquery_t *query, int tid)
{
LOG_DBG_SERVER(("Entering clientHandle()"));
FILE *f = NULL;

httpresp_t resp;
char str[1024];
int i,j;
char ext[17];
char out[MAX_FILE_SIZE+1];
unsigned long int filesize;

	resp.code = 200;
	resp.location[0]='\0';
	resp.ct[0]='\0';
	resp.cookies[0]='\0';

if(query->method)
LOG_DBG_SERVER(("Method : '%s'",query->method));
if(query->url)
LOG_DBG_SERVER(("URL : '%s'",query->url));
if(query->get)
LOG_DBG_SERVER(("Get : '%s'",query->get));
if(query->post)
LOG_DBG_SERVER(("Post : '%s'",query->post));
if(query->cookies)
LOG_DBG_SERVER(("Cookies : '%s'",query->cookies));

	if((strcmp(query->method,"GET")&&strcmp(query->method,"POST"))||query->url[0]=='\0')
	{
LOG_DBG_SERVER(("[%d] Invalid HTTP request (%d,%d,%d/%d/%d) ; closing connection",tid,strcmp(query->method,"GET"),strcmp(query->method,"POST"),query->url[0]!='\0',(strcmp(query->method,"GET")&&strcmp(query->method,"POST")),(strcmp(query->method,"GET")&&strcmp(query->method,"POST"))||query->url[0]=='\0'));
		return;
	}

	GET_GAME_DIR(str);
	sprintf(str,"%s/addons/w/www%s%s",str,query->url[0]=='/'?"":"/",query->url);

	if((query->url)[strlen(query->url)-1]=='/')
	{
		(str)[strlen(str)-1]='\0'; // remove '/' at the end
	}

	if(strstr(str,"../")) // hack attempt, display index page
	{
		str[0]='\0';
	}

	DIR *dirptr;

	dirptr = opendir(str);
	if(dirptr) // this is a directory
	{
		closedir(dirptr);
		if((query->url)[strlen(query->url)-1]!='/') // add '/' to location if client didn't send it
		{
			resp.code = 302;
			sprintf(resp.location,"%s/",query->url);
			sendHeaders(connfd,&resp,query,false);
		}
		else
			strcat(str,"/index.w");
	}
	
	if(resp.code==200 && (f=fopen(str,"rb"))==NULL)
	{
LOG_DBG_SERVER(("COULD NOT OPEN FILE : '%s' - errno=%d",str,errno));
		if(strcmp(query->url,"/") && strcmp(query->url,"/index.w"))
		{
			resp.code = 302;
			strcpy(resp.location,"/");
			sendHeaders(connfd,&resp,query,false);
		}
		else
		{
			resp.code = 404;
			strcpy(resp.ct,"text/html");
			sendHeaders(connfd,&resp,query,true);
			MYSOCK_WRITE(connfd,"<b>FATAL ERROR:</b> no index.w in root dir");
		}
	}
	else if(resp.code==200)
	{
		fclose(f);
		ext[0]='\0';
		for(i=strlen(str)-1,j=0;str[i]!='.'&&i>=0&&j<17;i--,j++)
			;
		if(i!=0&&str[i]=='.')
			strcpy(ext,str+i+1);

		if(!strcmp(ext,"w"))
			strcpy(resp.ct,"text/html");
		else if(!strcmp(ext,"css"))
			strcpy(resp.ct,"text/css");
		else if(!strcmp(ext,"jpg")||!strcmp(ext,"jpeg"))
			strcpy(resp.ct,"image/jpeg");
		else if(!strcmp(ext,"gif"))
			strcpy(resp.ct,"image/gif");
		else
			strcpy(resp.ct,"text/html");

		if(!strcmp(ext,"w"))
		{
			char parsed[MAX_FILE_SIZE+1];
			readFile(out,str,&filesize);
			parsed[0]='\0';
LOG_DBG_SERVER(("[%d] Parsing file",tid));
			parseNotScript(parsed,out,query,NULL);
LOG_DBG_SERVER(("[%d] Sending result",tid));
			sendHeaders(connfd,&resp,query,true);
			MYSOCK_WRITE(connfd,parsed);
		}
		else
		{
LOG_DBG_SERVER(("[%d] Direct send",tid));
			sendHeaders(connfd,&resp,query,false);
			sendFile(connfd,str);
		}
	}


LOG_DBG_SERVER(("[%d] Finished for that request",tid));


	return;
}

void connectHandle(void *data)
{
char *input;
char buf[MYSOCK_BUFLEN+1];
int i,j;
int connfd;
int myid;
threaddata_t *tdata;

httpquery_t query;

	memset(buf,0,MYSOCK_BUFLEN+1);
	//memset(query.sendcook,0,MYSOCK_BUFLEN+1);

	query.currentmapname[0]='\0';
	query.sendcookies[0]='\0';
	query.contenttype[0]='\0';
	query.location[0]='\0';

	for(i=0;i<256;i++)
	{
		query.vars[i].name[0]='\0';
		query.vars[i].value[0]='\0';
	}

	tdata = (threaddata_t *) data;

	connfd = tdata->connfd;
	//myid = (int)(tdata->threadid);
	myid = MYTHREAD_WHOAMI();
	strcpy(query.clientip,tdata->clientip);

LOG_DBG_SERVER(("[%d] CONNECTION (%s) !",myid,query.clientip));

	if (waitInput(connfd,5)<1)
	{
LOG_DBG_SERVER(("[%d,%d] No data received ; closing connection",myid,connfd));

		shutdown(connfd,2);
		MYSOCK_CLOSE(connfd);
		return;
	}

	char tmp[1025];

	int rcv = MYSOCK_READ(connfd,tmp,1024);

	if(rcv<=0)
	{
LOG_DBG_SERVER(("[%d,%d] READ error",myid,connfd));
		shutdown(connfd,2);
		MYSOCK_CLOSE(connfd);
		return;
	}

	tmp[rcv]='\0';

LOG_DBG_SERVER(("Allocating %d bytes for input string",rcv+1));
	input = (char *)malloc(rcv+1);
LOG_DBG_SERVER(("Mem location : 0x%X",input));

	// take out those crappy \r sent by w*nd0ws browsers
	for(j=0,i=0;tmp[i];i++)
		if(tmp[i]!='\r')
			input[j++]=tmp[i];

	input[j]='\0';

LOG_DBG_SERVER(("[%d,%d] Finished reading %d bytes request",myid,connfd,j));
LOG_DBG_SERVER(("Input from %s :\n----\n%s\n----",query.clientip,input));

	// get METHOD field (max 10 chars)
	for(i=0;/*i<=MYSOCK_BUFLEN && */input[i]!=' ' && input[i]!='\0' && input[i]!='\n' && i<10;i++)
		query.method[i]=input[i];
	query.method[i]='\0';

LOG_DBG_SERVER(("Method : %s",query.method));

	for(;input[i]==' ';i++)
		;

	// get URL field (max 256 chars)
	for(j=0;/*i<=MYSOCK_BUFLEN && */input[i]!=' ' && input[i]!='\0' && input[i]!='\n' && input[i]!='?' && i<256;i++,j++)
		query.url[j]=input[i];
	query.url[j]='\0';

LOG_DBG_SERVER(("URL : %s",query.url));

	// get GET data (dynamically allocated)
	query.get = NULL;
	if(input[i]=='?')
	{
		for(j=0;input[i+j]!='\n'&&input[i+j]!=' '&&input[i+j]!='\0';j++)
			;
LOG_DBG_SERVER(("Allocating %d bytes for GET",j+1));
		query.get = (char *) malloc(j+1);
LOG_DBG_SERVER(("Mem location : 0x%X",query.get));
		i++;
		for(j=0;/*i<=MYSOCK_BUFLEN && */input[i]!=' ' && input[i]!='\0' && input[i]!='\n';i++,j++)
			query.get[j]=input[i];
		query.get[j]='\0';
	}
LOG_DBG_SERVER(("GET : %s",query.get?query.get:"(null)"));

	// get cookies... gonna be funny =/
	// "\nCookie: var=val;"
	query.cookies = NULL;
	for(i=0;(size_t)i<strlen(input);i++)
	{
		if(!strncmp(input+i,"\nCookie: ",9))
		{
			for(j=0;input[i+j+9]!='\n';j++)
				;
			if(query.cookies == NULL)
			{
LOG_DBG_SERVER(("Allocating %d bytes for cookies",j+1));
				query.cookies = (char *) malloc(j+1);
LOG_DBG_SERVER(("Mem location : 0x%X",query.cookies));
				query.cookies[0]='\0';
			}
			else
			{
LOG_DBG_SERVER(("Reallocating %d bytes for cookies",j+1));
				query.cookies = (char *) realloc(query.cookies,j+1);
LOG_DBG_SERVER(("Mem location : 0x%X",query.cookies));
			}
			i+=9;
			do
			{
LOG_DBG_SERVER(("Extracting cookie..."));
				if(input[i]==';')
					i+=2;
				for(j=0;input[i]&&input[i]!=';'&&input[i]!='\n';j++,i++)
					buf[j]=input[i];
				buf[j]='\0';
LOG_DBG_SERVER(("Cookie found : '%s'",buf));
				if(strlen(query.cookies))
					strcat(query.cookies,"&");
				strcat(query.cookies,buf);
			}
			while(input[i]==';')
				;
		}
	}
LOG_DBG_SERVER(("Cookies : %s (%d)",query.cookies?query.cookies:"(null)",query.cookies?strlen(query.cookies):0));

	// get POST data (dynamically alloced)
	query.post = NULL;
	if(!strcmp(query.method,"POST"))
	{
LOG_DBG_SERVER(("POST spotted ! search in %d",strlen(input)));
		char clbuf[11];

		for(i=0;(size_t)i<strlen(input)-17;i++)
		{
			if(!strnicmp(input+i,"\nContent-Length: ",17))
			{
LOG_DBG_SERVER(("Content-length spotted"));
				i+=17;
				for(j=0;input[i]!='\n'&&input[i];i++,j++)
					clbuf[j]=input[i];
				clbuf[j]='\0';
LOG_DBG_SERVER(("POST: length=%d ; allocating...",atoi(clbuf)));
				query.post = (char *) malloc(atoi(clbuf)+1);
LOG_DBG_SERVER(("Mem location : 0x%X",query.post));
				break;
			}
		}

		if(!query.post)
		{
LOG_DBG_SERVER(("CONTENT-LENGTH NOT FOUND"));
		}
		else
		{
			// new way to get POST
			for(;input[i]&&!(input[i]=='\n'&&input[i-1]=='\n');i++)
				;

			j=0;
			if(input[i+1])
			{
				i++;
				for(j=0;input[i]!='\0';i++,j++)
					query.post[j] = input[i];
			}
			query.post[j]='\0';

			if(j<atoi(clbuf))
			{
			// here we have to read from our socket again in order to get pending data
LOG_DBG_SERVER(("Remaining POST data in socket buffer ! (%d/%d read :'%s') -- allocating...",j,atoi(clbuf),query.post));
				char *tmp2;
				tmp2 = (char *) malloc(atoi(clbuf)-j+1);
LOG_DBG_SERVER(("Mem location : 0x%X",tmp2));

				while(strlen(query.post)<(size_t)atoi(clbuf))
				{
					if (waitInput(connfd,5)<1)
					{
LOG_DBG_SERVER(("[%d,%d] No data (post) received ; closing connection",myid,connfd));
						free(input);
						free(tmp2);
						if(query.get)
							free(query.get);
						if(query.post)
							free(query.post);
						if(query.cookies)
							free(query.cookies);
						free(tdata);

						shutdown(connfd,2);
						MYSOCK_CLOSE(connfd);
						return;
					}
					rcv = MYSOCK_READ(connfd,tmp2,/*atoi(clbuf)-j*/1024);
					if(rcv<=0)
					{
LOG_DBG_SERVER(("[%d,%d] READ (post) error",myid,connfd));
						free(input);
						free(tmp2);
						if(query.get)
							free(query.get);
						if(query.post)
							free(query.post);
						if(query.cookies)
							free(query.cookies);
						free(tdata);
						shutdown(connfd,2);
						MYSOCK_CLOSE(connfd);
						return;
					}
					tmp2[rcv]='\0';
					strcat(query.post,tmp2);
LOG_DBG_SERVER(("Read intermediary POST data : %d bytes",strlen(tmp2)));
				}
LOG_DBG_SERVER(("Finished reading POST data"));
				free(tmp2);
			}
		}
	}

LOG_DBG_SERVER(("POST : %s",query.post?query.post:"(null)"));

	clientHandle(connfd,&query,myid);

LOG_DBG_SERVER(("Shutting down socket"));
	shutdown(connfd,2);

LOG_DBG_SERVER(("Closing socket"));
	MYSOCK_CLOSE(connfd);

LOG_DBG_SERVER(("Freeing input=%d",input));
	free(input);


LOG_DBG_SERVER(("Freeing get=%d",query.get));
	if(query.get)
		free(query.get);


LOG_DBG_SERVER(("Freeing post=%d",query.post));
	if(query.post)
		free(query.post);


LOG_DBG_SERVER(("Freeing cookies=%d",query.cookies));
	if(query.cookies)
		free(query.cookies);

LOG_DBG_SERVER(("Freeing tdata=%d",tdata));
	free(tdata);

LOG_DBG_SERVER(("TOTALLY DONE with this client"));
}

void cleanClose(int param)
{
	MYSOCK_STOP(sock);
	exit(0);
}

void readFile(char *filecontent, const char * filename,unsigned long int *filesize)
{
FILE *f;
int c;
//  char str[2];
unsigned long int pos;

LOG_DBG_SERVER(("reading file %s",filename));

	f = fopen(filename,"rb");
	if(f != NULL)
	{
		c = fgetc(f);
		pos=0;
		while(!feof(f))
		{
			if(c!='\r')
				filecontent[pos++]=c;
			c = fgetc(f);
		}
		filecontent[pos]='\0';
		if(filesize)
			*filesize=pos;
		fclose(f);
	}
}

void sendFile(int connfd, const char * filename)
{
#define FILESTEP 1024
	FILE *f;
	char str[FILESTEP+1];
	int i;

	unsigned long int sz = 0;

LOG_DBG_SERVER(("Sending '%s' to #%d",filename,connfd));

	f = fopen(filename,"rb");
	if(f != NULL)
	{
		str[1]='\0';
		while(!feof(f))
		{
			for(i=0;i<FILESTEP && !feof(f);i++,sz++)
				str[i]=fgetc(f);
			if(MYSOCK_WRITEBIN(connfd,str,feof(f)?i-1:i)==-1)
			{
LOG_DBG_SERVER(("FAILURE (%d) - errno: %d",connfd,errno));
				fclose(f);
				return;
			}
		}
		fclose(f);
	}
LOG_DBG_SERVER(("Finished '%s' for #%d - %d bytes",filename,connfd,sz));
}

bool formDecode(const char *field, const char *src, char *dest,bool decode)
{
int i,j;
char f[256];

LOG_DBG_PARSER(("Decode : '%s' in '%s' (%d)",field,src?src:"NULL",src?strlen(src):0));
	dest[0]='\0';

	if(src==NULL || strlen(src)==0)
	{
LOG_DBG_PARSER(("Not found (empty)"));
		return false;
	}

	sprintf(f,"%s=",field);
	memset(dest,0,strlen(src));

	for(i=0;i<(signed)(strlen(src)-strlen(f));i++)
	{
		if(!strncmp(f,src+i,strlen(f)))
		{
			for(;src[i]&&src[i]!='=';i++)
				;
			for(i++,j=0;src[i]!='&' && src[i];i++)
			{
				if(decode)
				{
					switch(src[i])
					{
					case '+': dest[j++]=' '; break;
					case '%': dest[j++]=IntFromHex((char*)src+(++i)); i++; break;
					default : dest[j++]=src[i]; break;
					}
				}
				else
				{
					dest[j++]=src[i];
				}
			}
LOG_DBG_PARSER(("Found : '%s'",dest));
			return true;
		}
		for(;src[i]!='&'&&src[i];i++)
			;
	}

LOG_DBG_PARSER(("Not found"));
	return false;
}

// Now, a subroutine that unescapes escaped characters.
int IntFromHex(char *pChars)
{
int Hi;        // holds high byte
int Lo;        // holds low byte
int Result;    // holds result

	// Get the value of the first byte to Hi

	Hi = pChars[0];
	if ('0' <= Hi && Hi <= '9')
		Hi -= '0';
	else if ('a' <= Hi && Hi <= 'f')
		Hi -= ('a'-10);
	else if ('A' <= Hi && Hi <= 'F')
		Hi -= ('A'-10);


	// Get the value of the second byte to Lo
	Lo = pChars[1];
	if ('0' <= Lo && Lo <= '9')
		Lo -= '0';
	else if ('a' <= Lo && Lo <= 'f')
		Lo -= ('a'-10);
	else if ('A' <= Lo && Lo <= 'F')
		Lo -= ('A'-10);

	Result = Lo + (16 * Hi);

	return (Result);
}

void setAddrIn(MYSOCK_ADDRTYPE_IN *sin,bool quiet)
{
	if(!strcmp(CVAR_GET_STRING("ip"),"localhost"))
	{
		sin->sin_addr.s_addr   = inet_addr("0.0.0.0");
	}
	else
	{
		if(!quiet)
			LOG_MESSAGE(PLID,"Listening on address : %s",CVAR_GET_STRING("ip"));
		sin->sin_addr.s_addr   = inet_addr(CVAR_GET_STRING("ip"));

		if((signed)sin->sin_addr.s_addr == -1)
		{ // host not found ; might be a dns name
			struct hostent *he;
			char ip[16];

			he = gethostbyname(CVAR_GET_STRING("ip"));

			if(he==NULL)
			{
				if(!quiet)
					LOG_MESSAGE(PLID,"ERROR: could not determine IP address from cvar : '%s'",CVAR_GET_STRING("ip"));
			}
			else
			{
				sprintf(ip,"%d.%d.%d.%d",(he->h_addr_list[0][0]+256)%256,(he->h_addr_list[0][1]+256)%256,(he->h_addr_list[0][2]+256)%256,(he->h_addr_list[0][3]+256)%256);
				if(!quiet)
					LOG_MESSAGE(PLID,"Found IP for this name : %s",ip);
				sin->sin_addr.s_addr   = inet_addr(ip);
			}

		}
	}

	if((signed)sin->sin_addr.s_addr == -1)
	{
		if(!quiet)
			LOG_MESSAGE(PLID,"No valid IP found ; trying to listen on 0.0.0.0");
		sin->sin_addr.s_addr   = inet_addr("0.0.0.0");
	}

	if(!quiet)
		LOG_MESSAGE(PLID,"Listening on port : %d",(int)CVAR_GET_FLOAT("port"));

	sin->sin_family        = AF_INET;
	sin->sin_port          = htons((int)CVAR_GET_FLOAT("port"));
}
