#include <extdll.h>
#include <sdk_util.h>
#include <meta_api.h>

#include "webmod.h"
#include "w_os.h"


// creating threads
#ifdef WIN32

int MYTHREAD_CREATE(THREAD_T *tid, void (*func)(void *), void *data)
{
HANDLE ret;

	LOG_DBG_OSDEP(("[OSDEP] MYTHREAD_CREATE"));

	ret=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) func, data, 0, tid);

	if(ret==NULL)
	{
		LOG_MESSAGE(PLID,"Failure starting thread !");
		exit(1);
	}

	SetThreadPriority(ret,THREAD_PRIORITY_LOWEST);

	return(ret==NULL);
}

#else

int MYTHREAD_CREATE(THREAD_T *tid, void (*func)(void *), void *data)
{
int ret;
struct sched_param tattr;

	LOG_DBG_OSDEP(("[OSDEP] MYTHREAD_CREATE"));

	ret=pthread_create(tid, NULL, (void *(*)(void*)) func, data);

	LOG_DBG_OSDEP(("[OSDEP] Thread created"));

	if(ret != 0)
	{
		LOG_MESSAGE(PLID,"Failure starting thread: %s", strerror(ret));
		exit(1);
	}

	LOG_DBG_OSDEP(("[OSDEP] Detaching"));

	ret=pthread_detach(*tid);

	LOG_DBG_OSDEP(("[OSDEP] Detached"));

	if(ret != 0)
	{
		LOG_MESSAGE(PLID,"Failure detaching thread: %s", strerror(ret));
		exit(1);
	}

	LOG_DBG_OSDEP(("[OSDEP] Setting priority"));

	memset(&tattr,0x0,sizeof(tattr));
	tattr.sched_priority = sched_get_priority_min(SCHED_OTHER);
	ret=pthread_setschedparam(*tid,SCHED_OTHER,&tattr);

	LOG_DBG_OSDEP(("[OSDEP] Priority set"));

	return(ret);
}

#endif






// waiting for data from sockets
#ifdef WIN32

int waitInput(int connfd,int secs)
{
fd_set rfds;
struct timeval tv;
int res;

	LOG_DBG_OSDEP(("[OSDEP] waitInput"));

	FD_ZERO(&rfds);
	FD_SET(connfd,&rfds);

	tv.tv_sec=secs;
	tv.tv_usec=0;

	res = select(1,&rfds,NULL,NULL,&tv);

	LOG_DBG_OSDEP(("[OSDEP] waitInput : res=%d",res));

	return res;
}

#else

#include <sys/poll.h>
int waitInput(int connfd,int secs=5)
{
struct pollfd pfd;
int res;

	LOG_DBG_OSDEP(("[OSDEP] waitInput"));

// we don't use select() cause under linux it seems to block if data already available
// (and poll() doesn't exist under win32)

	pfd.fd = connfd;
	pfd.events = POLLIN | POLLPRI;

	res = poll(&pfd,1,1000*secs);

	LOG_DBG_OSDEP(("[OSDEP] waitInput : res=%d",res));

	return res;
}

#endif






// mutex
#ifdef WIN32

int MYMUTEX_INIT(MYMUTEX_T *mutex)
{
	InitializeCriticalSection(mutex);
	return(THREAD_OK);
}

int MYMUTEX_LOCK(MYMUTEX_T *mutex)
{
	EnterCriticalSection(mutex);
	return(THREAD_OK);
}

int MYMUTEX_UNLOCK(MYMUTEX_T *mutex)
{
	LeaveCriticalSection(mutex);
	return(THREAD_OK);
}

#else

int MYMUTEX_INIT(MYMUTEX_T *mutex)
{
int ret;

	ret=pthread_mutex_init(mutex, NULL);

	if(ret!=THREAD_OK)
	{
		LOG_MESSAGE(PLID,"mutex_init failed: %s", strerror(ret));
		exit(1);
	}

	return(ret);
}

int MYMUTEX_LOCK(MYMUTEX_T *mutex)
{
int ret;

	ret=pthread_mutex_lock(mutex);

	if(ret!=THREAD_OK)
	{
		LOG_MESSAGE(PLID,"mutex_lock failed: %s", strerror(ret));
		exit(1);
	}

	return(ret);
}

int MYMUTEX_UNLOCK(MYMUTEX_T *mutex)
{
int ret;

	ret=pthread_mutex_unlock(mutex);

	if(ret!=THREAD_OK)
	{
		LOG_MESSAGE(PLID,"mutex_unlock failed: %s", strerror(ret));
		exit(1);
	}

	return(ret);
}

#endif

