#ifndef _W_OS_H_
#define _W_OS_H_

#include <time.h>



// mutex ////////////////////////////////////
#ifdef WIN32
typedef CRITICAL_SECTION	MYMUTEX_T;
#else
typedef pthread_mutex_t		MYMUTEX_T;
#endif

int MYMUTEX_INIT(MYMUTEX_T *mutex);
int MYMUTEX_LOCK(MYMUTEX_T *mutex);
int MYMUTEX_UNLOCK(MYMUTEX_T *mutex);




// sockets ///////////////////////////////////
#ifdef WIN32 // win32 socket macros
// includes
#include <winsock.h>
// specific routines
#define MYSOCK_T SOCKET
#define MYSOCK_LENT int
#define MYSOCK_ADDRTYPE_IN SOCKADDR_IN
#define MYSOCK_ADDRTYPE SOCKADDR
#define MYSOCK_WRITE(sock,str) send(sock,str,strlen(str),0)
#define MYSOCK_WRITEBIN(sock,str,size) send(sock,str,size,0)
#define MYSOCK_CLOSE closesocket
#define MYSOCK_START() { WSADATA WSAData; WSAStartup(MAKEWORD(1,0), &WSAData); }
#define MYSOCK_STOP(sock) WSACleanup()
#define MYSOCK_READ(sock,buf,len) recv(sock,buf,len,0)
#define MYSOCK_SETOPT(sock,param) { char flag=1; setsockopt(sock,SOL_SOCKET,param,&flag,sizeof(flag)); }
#define MYSOCK_ACCEPT(sock,sin,len) accept(sock,(MYSOCK_ADDRTYPE*)&sin,len)
#define snprintf _snprintf
#ifndef sleep
#define sleep _sleep
#endif
#define SIGPIPE 0
#define ENOTSOCK WSAENOTSOCK
#define EADDRINUSE WSAEADDRINUSE
#define ELOOP WSAELOOP
#else // linux socket macros
// includes
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <unistd.h>
// specific routines
#define MYSOCK_T int
#define MYSOCK_LENT socklen_t
#define MYSOCK_WRITE(sock,str) write(sock,str,strlen(str))
#define MYSOCK_WRITEBIN(sock,str,size) write(sock,str,size)
#define MYSOCK_CLOSE close
#define MYSOCK_ADDRTYPE_IN struct sockaddr_in
#define MYSOCK_ADDRTYPE struct sockaddr
#define MYSOCK_START() { }
#define MYSOCK_STOP(sock) shutdown(sock,2)
#define MYSOCK_READ(sock,buf,len) read(sock,buf,len)
#define MYSOCK_SETOPT(sock,param) { int flag=1; setsockopt(sock,SOL_SOCKET,param,&flag,sizeof(flag)); }
#define MYSOCK_ACCEPT(sock,sin,len) accept(sock,(MYSOCK_ADDRTYPE*)&sin,len)
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#endif // end of specific declarations
// general os-independant macros
#define MYSOCK_NEW(sock) (sock = socket(AF_INET,SOCK_STREAM,0))
#define MYSOCK_BIND(sock,sin) bind(sock,(MYSOCK_ADDRTYPE*)&sin,sizeof(sin))
#define MYSOCK_LISTEN(sock,nb) listen(sock,nb)

#define MYSOCK_BUFLEN 8192





// thread ////////////////////////////////////
#ifdef WIN32
typedef	DWORD 		THREAD_T;
#define MYTHREAD_WHOAMI() GetCurrentThreadId()
#else
#include <pthread.h>
typedef	pthread_t 	THREAD_T;
#define MYTHREAD_WHOAMI() pthread_self()
#endif

int MYTHREAD_CREATE(THREAD_T *tid, void (*func)(void *), void *data);



// socket waiting ////////////////////////////
int waitInput(int connfd,int secs);


#endif // _W_OS_H_

