#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include <time.h>

#include <extdll.h>
#include <sdk_util.h>
#include <meta_api.h>
#include <cbase.h>

#include "webmod.h"
#include "msg.h"

cvar_t cvar_version = { "w_version", VVERSION, FCVAR_SERVER, VVERSIONF, NULL };

bool threadok = false;
int fram;
player_t players[33];
time_t up_since;
team_t teams[32];
debug_t debug;


#ifdef MALLOC_DEBUG
void memStats(void)
{
	RM_STAT;
}

void memTests(void)
{
	RM_TEST;
}
#endif


void wInit (void)
{

	REG_SVR_COMMAND("w_debug",setDebug);
#ifdef MALLOC_DEBUG
	REG_SVR_COMMAND("ms",memStats);
	REG_SVR_COMMAND("mt",memTests);
#endif
	CVAR_REGISTER (&cvar_version);

	debug.server = false;
	debug.parser = false;
	debug.api = false;
	debug.os = false;

	for(int i=0;i<33;i++)
	{
		player_t *pPlayer = INDEXPLAYER(i);
		pPlayer->pEdict = NULL;
		pPlayer->index = -i;
		pPlayer->ingame = false;
		pPlayer->connected = false;
		pPlayer->bomb = false;
		pPlayer->vip = false;
	}

	up_since = time(NULL);
}


BOOL handleClientConnect (edict_t * pEntity, const char *pszName, const char *pszAddress, char szRejectReason[128])
{

	int i;
	int index = ENTINDEX(pEntity);

LOG_DBG_API(("Client connection : %s from %s ; index=%d",pszName,pszAddress,index));

	player_t *pPlayer = INDEXPLAYER(index);
    pPlayer->connected = true;
	pPlayer->pEdict = pEntity;

	for(i=0;pszAddress[i]&&pszAddress[i]!=':'&&i<15;i++)
		pPlayer->ip[i]=pszAddress[i];

	pPlayer->ip[i]='\0';
	pPlayer->connected_since = time(NULL);
    
	pPlayer->bomb = false;
	pPlayer->vip = false;

	RETURN_META_VALUE (MRES_HANDLED, TRUE);
}

void clientPutInServer( edict_t *pEntity ) {  
	int index = ENTINDEX(pEntity);
LOG_DBG_API(("Client putInServer : index=%d",index));
	player_t *pPlayer = INDEXPLAYER(index);
	pPlayer->pEdict = pEntity;
	pPlayer->index = index;
	pPlayer->ingame = true;
	pPlayer->bomb = false;
	pPlayer->vip = false;
	pPlayer->ingame_since = time(NULL);
	RETURN_META(MRES_IGNORED);
}

void clientDisconnect( edict_t *pEntity ) {
	player_t *pPlayer = ENTPLAYER(pEntity);
	pPlayer->ingame = false;
	pPlayer->connected = false;
	pPlayer->bomb = false;
	pPlayer->vip = false;
	RETURN_META(MRES_IGNORED);
}


void setDebug()
{
	char debugs[128];

	switch(CMD_ARGC())
	{
	case 1:
		debugs[0]='\0';
		if(debug.server)
			strcat(debugs," server");
		if(debug.parser)
			strcat(debugs," parser");
		if(debug.api)
			strcat(debugs," api");
		if(debug.os)
			strcat(debugs," os");
		if(debugs[0]=='\0')
			strcpy(debugs," (none)");
		LOG_MESSAGE(PLID,"Current debug modes :%s",debugs);
		break;
	case 2:
		if(!strcmp(CMD_ARGV(1),"server"))
			debug.server = true;
		else if(!strcmp(CMD_ARGV(1),"parser"))
			debug.parser = true;
		else if(!strcmp(CMD_ARGV(1),"api"))
			debug.api = true;
		else if(!strcmp(CMD_ARGV(1),"os"))
			debug.os = true;
		else if(!strcmp(CMD_ARGV(1),"off"))
		{
			debug.server = false;
			debug.parser = false;
			debug.api = false;
			debug.os = false;
		}
		else if(!strcmp(CMD_ARGV(1),"all"))
		{
			debug.server = true;
			debug.parser = true;
			debug.api = true;
			debug.os = true;
		}
		else
			LOG_MESSAGE(PLID,"Syntax : w_debug <off|all|server|parser|api|os>");
		break;
	default:
		LOG_MESSAGE(PLID,"Syntax : w_debug <off|all|server|parser|api|os>");
		break;
	}
}


void myFrame()
{
int i;

	if(fram)
	{
		switch(fram)
		{
		case 1:

LOG_DBG_API(("Initializing player structures"));
			for(i=0;i<33;i++)
			{
				player_t *pPlayer = INDEXPLAYER(i);
				pPlayer->pEdict = NULL;
				pPlayer->index = -i;
				pPlayer->ingame = false;
				pPlayer->connected = false;
			}
			memset(teams,0,sizeof(teams));
			break;
		case 17:
			if(!threadok)
			{
LOG_DBG_SERVER(("Starting thread..."));
				startThread();
				threadok = true;
			}
			break;
		case 19:
			fram = -1;
			break;
		}
		fram++;
	}

	RETURN_META (MRES_HANDLED);
}


void UTIL_LogPrintf (char *fmt, ...)
{
va_list argptr;
static char string[1024];

	va_start (argptr, fmt);
	vsprintf (string, fmt, argptr);
	va_end (argptr);

	ALERT (at_logged, "%s", string);
}

void ServerActivate (edict_t * pEdictList, int edictCount, int clientMax)
{
	fram=1;
	RETURN_META (MRES_HANDLED);
}


char* UTIL_VarArgs( char *format, ... )
{
va_list argptr;
static char string[16385]; // pretty big isn't it ?

	va_start (argptr, format);
	vsprintf (string, format, argptr);
	va_end (argptr);

	return string;
}
