#ifndef _WEBMOD_H_
#define _WEBMOD_H_

#ifdef MALLOC_DEBUG
#include "rmalloc.h"
#endif

#include "w_os.h"

#define VDATE			"2003/05/19"
#define VVERSION		"0.47"
#define VVERSIONF		0.47

#define VNAME		"WebMod"
#define VAUTHOR		"dJeyL"
#define VURL        "http://djeyl.net/w"
#define VLOGTAG		"W"

struct player_t {
	edict_t*	pEdict;
	int			index;
	bool		ingame;
	bool		connected;
	time_t		connected_since;
	time_t		ingame_since;

	char		ip[16];
	char		*team;
	int			team_id;
	int			deaths;
	bool		bomb;
	bool		vip;
};
extern player_t players[33];
extern int fram;
extern bool freebsd;

struct team_t {
	char	name[65];
	int		score;
};
extern team_t teams[32];

typedef struct s_debug {
	bool server; // http server
	bool parser; // w scripts parser
	bool api; // half-life & metamod api
	bool os; // os specific functions
} debug_t;
extern debug_t debug;

extern time_t up_since;


typedef struct s_stralloc {
	size_t alloced;
	char *str;
} stralloc_t;

#define INDEXPLAYER(a) (&players[a])
#define ENTPLAYER(a) (&players[ENTINDEX(a)])
#define MYINDEXENT(a) (players[a].pEdict)

void clientPutInServer( edict_t *pEntity );
void clientDisconnect( edict_t *pEntity );

void startThread(void);

void wInit ();
void wActivate ();
BOOL handleClientConnect (edict_t * pEntity, const char *pszName, const char *pszAddress, char szRejectReason[128]);
void myFrame(void);
void UTIL_LogPrintf (char *fmt, ...);

#define MAX_FILE_SIZE 16384

typedef struct httpresp_s {
  int code;
  char location[256];
  char ct[256];
  char cookies[MYSOCK_BUFLEN+1];
} httpresp_t;

typedef struct s_var {
	char name[33];
	char value[513];
} var_s;

typedef struct s_httpquery {
  char method[11];
  char clientip[16];
  char url[257];
  char *get;
  char *post;
  char *cookies;
  var_s vars[256];
  char currentmapname[257];
  char sendcookies[MYSOCK_BUFLEN+1];
  char contenttype[257];
  char location[257];
} httpquery_t;

void connectHandle(void *data);
int sendHeaders(int connfd, httpresp_t *resp, httpquery_t *query);
void clientHandle(int connfd, httpquery_t *query, int tid);
void cleanClose(int param);
void sendFile(int connfd, const char * filename);
void theThread(void *data);
bool formDecode(const char *field, const char *src, char *dest,bool decode);
int IntFromHex(char *pChars);
void readFile(char *filecontent, const char * filename,unsigned long int *filesize);

void ServerActivate (edict_t * pEdictList, int edictCount, int clientMax);

int waitInput(int connfd, int secs=5);

typedef struct s_threaddata {
	THREAD_T threadid;
	int connfd;
	char clientip[16];
}
threaddata_t;

char* UTIL_VarArgs( char *format, ... );

bool charInString(const char c, const char *str);
void parseScript(char *parsed,const char *stp,httpquery_t *query,player_t *pPlayer);
void parseNotScript(char *parsed,const char *stp,httpquery_t *query,player_t *pPlayer);
bool getVar(char *out, char *in, httpquery_t *query,player_t *pPlayer);

bool setVar(const char *name, const char *value, httpquery_t *query);

void setAddrIn(MYSOCK_ADDRTYPE_IN *sin,bool quiet);

bool execFunction(char *parsed,const char *fname, httpquery_t *query, player_t *pPlayer, const char params[16][256]);

void handleBinaryOperand(char *parsed, char why, const char *first, const char *second, httpquery_t *query,player_t *pPlayer);


void setDebug(void);

#define LOG_DEBUG(msg) LOG_MESSAGE(PLID,UTIL_VarArgs msg)
#define LOG_DBG_API(msg) {if(debug.api)LOG_DEBUG(msg);}
#define LOG_DBG_SERVER(msg) {if(debug.server)LOG_DEBUG(msg);}
#define LOG_DBG_PARSER(msg) {if(debug.parser)LOG_DEBUG(msg);}
#define LOG_DBG_OSDEP(msg) {if(debug.os)LOG_DEBUG(msg);}


// P2 interface
#define P2IF 2
struct p2if_t {
        unsigned int version;
        bool (*isKnownName)(const char *zeName);
        int (*getKnownPlayers)(void);
        bool (*isKnownID)(int id);
};

#endif // _WEBMOD_H_
